// Copyright (c) 2021, Kato Shinya. All rights reserved.
// Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

// Project imports:
import 'package:one_trust_geo_location/src/response/response.dart';
import 'package:one_trust_geo_location/src/response/status.dart';

/// This class represents the response object from the Geo Location API.
///
/// This class contains an estimate of the location. Note that the location information
/// included in the response returned from this API is approximate and not accurate.
///
/// **_Example:_**
///
/// ```dart
/// void main() async {
///   // It's very easy to call api with get function.
///   // It has no class so it's better to set alias 'location' like below.
///   final response = await location.get();
///
///   if (response.status.isNotOk) {
///     // Do something when it's client or server error.
///     if (response.status.isClientError) {
///       return;
///     } else if (response.status.isServerError) {
///       return;
///     } else {
///       return;
///     }
///   }
///
///   print(response.country);
///   print(response.latitude);
///   print(response.longitude);
/// }
/// ```
class GeoLocationResponse extends Response {
  /// Returns the new instance of [GeoLocationResponse] based on arguments.
  GeoLocationResponse.from({
    required int statusCode,
    required String reasonPhrase,
    required Map<String, String> headers,
    required this.country,
    required this.state,
    required this.stateName,
    required this.zipcode,
    required this.timezone,
    required this.latitude,
    required this.longitude,
    required this.city,
    required this.continent,
  }) : super.from(
          status: Status.from(
            code: statusCode,
            reasonPhrase: reasonPhrase,
          ),
          headers: headers,
        );

  /// The country
  final String country;

  /// The state
  final String state;

  /// The state name
  final String stateName;

  /// The zip code
  final String zipcode;

  /// The time zone
  final String timezone;

  /// The latitude
  final String latitude;

  /// The longitude
  final String longitude;

  /// The city
  final String city;

  /// The continent
  final String continent;

  @override
  String toString() {
    return 'GeoLocationResponse(country: $country, state: $state, stateName: $stateName, zipcode: $zipcode, timezone: $timezone, latitude: $latitude, longitude: $longitude, city: $city, continent: $continent)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is GeoLocationResponse &&
        other.country == country &&
        other.state == state &&
        other.stateName == stateName &&
        other.zipcode == zipcode &&
        other.timezone == timezone &&
        other.latitude == latitude &&
        other.longitude == longitude &&
        other.city == city &&
        other.continent == continent;
  }

  @override
  int get hashCode {
    return country.hashCode ^
        state.hashCode ^
        stateName.hashCode ^
        zipcode.hashCode ^
        timezone.hashCode ^
        latitude.hashCode ^
        longitude.hashCode ^
        city.hashCode ^
        continent.hashCode;
  }
}
